use crate::client::ONLINE_USERS;
use crate::events::{notify_all_clients_to_update_online_users, Event, CLIENT_SENDERS};
use crate::handles::handshake::HOST_ENABLED_ID_SET;
use crate::handles::redis::update_client_redis_data;
use crate::typing::message_typed::ClientMessageData;
use crate::utils::json_utils::get_current_timestamp;
use dashmap::DashMap;
use lazy_static::lazy_static;
use std::time::Duration;
use tokio::sync::mpsc;
use tokio::sync::mpsc::UnboundedSender;
use tokio::time::sleep;

lazy_static! {
    pub static ref refuse_procedure_map: DashMap<String, UnboundedSender<()>> = DashMap::new();
    pub static ref channel_hangup_procedure_map: DashMap<String, UnboundedSender<()>> =
        DashMap::new();
}

async fn send_inside_message(
    target_sender: &(String, u128),
    event_sender: &UnboundedSender<Event>,
    json_message: String,
    from_id: &String,
) {
    println!("发送给用户id {} 的消息 {}", from_id, json_message);
    if let Err(e) = event_sender.send(Event::SendClientMessage(
        target_sender.clone(),
        json_message,
        false,
    )) {
        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
    } else {
        println!("发送给用户id {} 独立消息成功", from_id);
    }
}

pub async fn handle_agora_call(
    client_message_data: &ClientMessageData,
    from_id: &String,
    event_sender: &UnboundedSender<Event>,
    connection_time: &u128,
) {
    let target_sender_which = (from_id.to_string(), connection_time.clone());
    // 对于非复杂性数据，不要使用serde_json，直接用format!宏即可
    match client_message_data.msg_type.as_str() {
        // 取消呼叫
        "CancelCall" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端取消呼叫 取消呼叫组: {} 呼叫方id {}",
                calling_to_id,
                from_id.to_string()
            );
            // 直接发送CmdHangup消息给当前用户
            let hangup_message_json = serde_json::json!({
                "msgType": "CmdCancelCall",
                "fromID": "0",
                "fromName": "Server",
                "toID": from_id,
                "msgData": {
                    "channelId": "",
                    "rtcToken": ""
                }
            })
            .to_string();
            if let Err(e) = event_sender.send(Event::SendClientMessage(
                target_sender_which.clone(),
                hangup_message_json,
                false,
            )) {
                println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
            }
            let user_status = { ONLINE_USERS.get(from_id) };
            // 对同一个channelID的组，发送挂断消息
            if let Some(user_status) = user_status {
                // 将获取的字符串用逗号拆分，并存储为 Vec<String>
                let mut the_current_user_data: Vec<String> =
                    user_status.split(',').map(|s| s.to_string()).collect();
                // 如果当前用户channelID为空，则中断后续操作，并且直接重新赋值即可
                if the_current_user_data[1].is_empty() {
                    the_current_user_data[1] = "".to_string();
                    the_current_user_data[0] = "idle".to_string();
                    the_current_user_data[6] = "0".to_string();
                    let data_joined = the_current_user_data.join(",");
                    drop(user_status);
                    drop(the_current_user_data);
                    ONLINE_USERS.insert(from_id.to_string(), data_joined.to_string());
                    let from_id_clone = from_id.clone();
                    tokio::spawn(async move {
                        if let Err(e) = update_client_redis_data(&from_id_clone, data_joined).await
                        {
                            println!("更新redis数据失败：{:?}", e);
                        };
                    });
                    tokio::spawn(async move {
                        notify_all_clients_to_update_online_users().await;
                    });
                    return;
                }
                // 拆分toId为其他用户id，并且获取其数据信息
                let all_to_hangup_id_vec: Vec<String> =
                    calling_to_id.split(',').map(|s| s.to_string()).collect();
                for to_hangup_id in all_to_hangup_id_vec {
                    // 取消拒绝接听的线程队列
                    if let Some(ref_procedure_sender) =
                        refuse_procedure_map.get(&to_hangup_id.to_string())
                    {
                        if let Err(e) = ref_procedure_sender.send(()) {
                            println!("取消拒绝接听的线程队列失败：{:?}", e);
                        } else {
                            println!("取消拒绝接听的线程队列成功");
                        }
                    }
                    let the_other_caller = { ONLINE_USERS.get(&to_hangup_id) };
                    // 获取当前用户状态信息成功后，再进行额外操作
                    if let Some(ref the_other_caller_data) = the_other_caller {
                        let mut the_other_caller_data: Vec<String> = the_other_caller_data
                            .split(',')
                            .map(|s| s.to_string())
                            .collect();
                        // 提前释放
                        drop(the_other_caller);
                        // 如果处于同一频道，则发送挂断信息，
                        if the_other_caller_data[1] == the_current_user_data[1] {
                            let hangup_message_json = serde_json::json!({
                                "msgType": "CmdHangup",
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": to_hangup_id,
                                "msgData": {
                                    "channelId": "",
                                    "rtcToken": ""
                                }
                            })
                            .to_string();
                            // 找到对应sender，发送CmdHangup消息
                            let target_sender = {
                                let mut iter = CLIENT_SENDERS.iter();
                                iter.find(|entry| entry.key().0 == *calling_to_id)
                                    .map(|entry| entry.key().clone())
                            };
                            if let Some(target_sender) = target_sender {
                                if let Err(e) = event_sender.send(Event::SendClientMessage(
                                    target_sender,
                                    hangup_message_json,
                                    false,
                                )) {
                                    println!("发送给用户id {} 独立消息失败：{:?}", to_hangup_id, e);
                                } else {
                                    // 修改对应sender的数据
                                    the_other_caller_data[0] = "idle".to_string();
                                    the_other_caller_data[1] = "".to_string();
                                    the_other_caller_data[6] = "0".to_string();
                                    let all_data_joined =
                                        the_other_caller_data.join(",").to_string();
                                    println!("step - 0");
                                    drop(the_other_caller_data);
                                    ONLINE_USERS.insert(
                                        to_hangup_id.to_string(),
                                        all_data_joined.to_string(),
                                    );
                                    println!("step - 1");
                                    tokio::spawn(async move {
                                        if let Err(e) =
                                            update_client_redis_data(&to_hangup_id, all_data_joined)
                                                .await
                                        {
                                            println!("更新redis数据失败：{:?}", e);
                                        };
                                    });
                                }
                            } else {
                                println!("未找到对应的sender");
                            }
                        }
                    }
                }
                println!("step - 1.5");
                // 要求所有在线用户更新数据
                the_current_user_data[1] = "".to_string();
                the_current_user_data[0] = "idle".to_string();
                the_current_user_data[6] = "0".to_string();
                let the_current_data_joined = the_current_user_data.join(",").to_string();
                drop(user_status);
                drop(the_current_user_data);
                println!("step - 1.7");
                ONLINE_USERS.insert(from_id.to_string(), the_current_data_joined.to_string());
                let from_id_clone = from_id.to_string();
                println!("step - 2.1");
                tokio::spawn(async move {
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, the_current_data_joined).await
                    {
                        println!("更新redis数据失败：{:?}", e);
                    };
                });
                tokio::spawn(async move {
                    println!("更新在线用户列表 Cancel-Call");
                    notify_all_clients_to_update_online_users().await;
                });
                println!("step - 2.2");
            }
        }
        // 对于通话类消息
        "Call" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端呼叫 呼叫方id {} 呼叫组id {}",
                from_id, calling_to_id
            );
            // 必须要指定呼叫对象
            if calling_to_id.is_empty() {
                let error_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "请指定呼叫对象", "toID": "{}"}}"#,
                    &from_id
                );
                send_inside_message(&target_sender_which, &event_sender, error_json, &from_id)
                    .await;
                return;
            }
            // 不允许自己跟自己打电话
            if calling_to_id.eq(from_id) {
                let error_self_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "不能给自己打电话", "toID": "{}"}}"#,
                    &from_id
                );
                send_inside_message(
                    &target_sender_which,
                    &event_sender,
                    error_self_json,
                    &from_id,
                )
                .await;
                return;
            }
            let user_status = { ONLINE_USERS.get(from_id) };
            // 获取当前用户状态信息成功后，再进行额外操作
            if let Some(user_status) = user_status {
                // 将获取的字符串用逗号拆分
                let mut the_user_data: Vec<&str> = user_status.split(',').collect();
                // 获取当前用户channelId
                let channel_id_now = if the_user_data[1].is_empty() {
                    format!("{}-{}", from_id, get_current_timestamp())
                } else {
                    the_user_data[1].to_string()
                };
                // 注意一下，这里的to_id有可能是多个，需要做一下过滤处理
                let calling_to_id_vec: Vec<&str> = calling_to_id.split(',').collect();
                // 储存需要计入CmdRefuse线程用户
                let mut refuse_thread_users: Vec<String> = vec![];
                println!(
                    "当前用户channelId {} 呼叫方id集合是 {:?}",
                    channel_id_now, calling_to_id_vec
                );
                for calling_to_id in calling_to_id_vec {
                    // 1.检查目标用户是否在线
                    if !ONLINE_USERS.contains_key(calling_to_id) {
                        let json_string = format!(
                            r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方不在线","toID":"{}"}}"#,
                            &client_message_data.to_id
                        );
                        send_inside_message(
                            &target_sender_which,
                            &event_sender,
                            json_string,
                            &from_id,
                        )
                        .await;
                        continue;
                    }
                    let user_status = { ONLINE_USERS.get(calling_to_id) };
                    // 2.检查目标用户是否为空闲状态
                    if let Some(user_status) = user_status {
                        // 将获取的字符串用逗号拆分
                        let mut the_another_user_data: Vec<&str> = user_status.split(',').collect();
                        // 如果不为idle，不允许后续操作
                        if the_another_user_data[0] != "idle" {
                            let json_string = format!(
                                r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方正在呼叫中","toID":"{}"}}"#,
                                from_id
                            );
                            send_inside_message(
                                &target_sender_which,
                                &event_sender,
                                json_string,
                                &from_id,
                            )
                            .await;
                            continue;
                        }
                        // 3.如果为空闲状态，准备发送Call类型消息
                        let call_message_json = serde_json::json!({
                            "msgType": "Call",
                            "fromID": from_id,
                            "fromName": client_message_data.from_name,
                            "toID": calling_to_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                        .to_string();
                        // 通知被呼叫方的sender发送数据
                        let target_sender = {
                            CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == calling_to_id) // 使用 entry.key() 访问键
                                .map(|entry| entry.key().clone())
                        }; // 使用 entry.value() 访问值
                        if let Some(sender) = target_sender {
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                sender.clone(),
                                call_message_json.clone(),
                                false,
                            )) {
                                println!("发送给用户id {:?} 独立消息失败：{:?}", sender, e);
                                continue;
                            } else {
                                println!(
                                    "发送给用户id {} 独立消息成功 消息内容 {}",
                                    calling_to_id, call_message_json
                                );
                            }
                        } else {
                            println!("找不到对应的sender，无法发送客户端消息！");
                            continue;
                        }
                        println!("发送给用户id {} 独立消息成功", calling_to_id);
                        // 修改对应用户的状态数据
                        the_another_user_data[0] = "callin";
                        the_another_user_data[1] = channel_id_now.as_str();
                        the_another_user_data[6] = "0";
                        println!("step - 1");
                        let the_another_user_data_joined = the_another_user_data.join(",");
                        // 更新状态数据
                        drop(user_status);
                        ONLINE_USERS.insert(
                            calling_to_id.to_string(),
                            the_another_user_data_joined.clone(),
                        );
                        println!("step - 2");
                        let calling_id_c = calling_to_id.to_string();
                        tokio::spawn(async move {
                            // 修改redis数据
                            if let Err(e) = update_client_redis_data(
                                &calling_id_c,
                                the_another_user_data_joined.clone(),
                            )
                            .await
                            {
                                println!("更新redis数据失败：{:?} 用户id {}", e, calling_id_c);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                        println!("step - 3");
                        refuse_thread_users.push(calling_to_id.to_string());
                    }
                }
                println!(
                    "是否通知其他用户成功？ {} 然后发送CmdCall？",
                    refuse_thread_users.len()
                );
                // 如果成功发送了呼叫信息，则根据当前用户状态发送消息通知
                if refuse_thread_users.len() > 0 {
                    println!("发送给用户id {} 需要发送CmdCall", from_id);
                    // 如果当前用户本身为idle，则发送CmdCall
                    if the_user_data[0] == "idle" || the_user_data[0] == "calling" {
                        let cmd_call_message_json = serde_json::json!({
                            "msgType": "CmdCall",
                            "fromID": calling_to_id,
                            "fromName": client_message_data.from_name,
                            "toID": from_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                        .to_string();
                        // 更新状态数据
                        the_user_data[0] = if the_user_data[0] == "calling" {
                            "calling"
                        } else {
                            "callout"
                        };
                        the_user_data[1] = channel_id_now.as_str();
                        the_user_data[6] = "0";
                        // 在连接成功之后再设置为主持人，在此之前一律将所有用户的isHost数据段设置为0
                        let the_user_data_joined = the_user_data.join(",");
                        drop(user_status);
                        println!("step - 4");
                        ONLINE_USERS.insert(from_id.to_string(), the_user_data_joined.clone());
                        let from_id_c = from_id.to_string();
                        println!("step - 5");
                        tokio::spawn(async move {
                            // 修改redis数据
                            if let Err(e) =
                                update_client_redis_data(&from_id_c, the_user_data_joined).await
                            {
                                println!("更新redis数据失败：{:?} 用户id {}", e, from_id_c);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                        println!("step - 6");
                        send_inside_message(
                            &target_sender_which,
                            event_sender,
                            cmd_call_message_json,
                            &from_id,
                        )
                        .await;
                        tokio::spawn(async move {
                            // 最后广播用户更新
                            notify_all_clients_to_update_online_users().await;
                        });
                        // 开启定时器线程任务，如果所有用户20s内没有接听，则挂断，发送所有CmdRefuse数据并重置
                        for user_id in refuse_thread_users {
                            let current_user_status_data = { ONLINE_USERS.get(&user_id) };
                            if let Some(current_user_status_data) = current_user_status_data {
                                // 设置线程任务
                                let user_id_clone = user_id.clone();
                                let from_id = from_id.clone();
                                let event_sender = event_sender.clone();
                                let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                                refuse_procedure_map.insert(user_id, cancel_tx);
                                let user_data_cloned_current = current_user_status_data.clone();
                                tokio::spawn(async move {
                                    tokio::select! {
                                        _ = sleep(Duration::from_secs(20)) => {
                                            println!("20秒内没有接听，准备挂断通话");
                                            // 20秒内没有接听，准备挂断通话
                                            let mut current_user_data_vec: Vec<String> = user_data_cloned_current.split(',').map(|s| s.to_string()).collect();
                                            // 提前取出channelID，判断是否还有人在频道里面
                                            let current_channel_id = current_user_data_vec[1].clone();
                                            current_user_data_vec[0] = "idle".to_string();
                                            current_user_data_vec[1] = "".to_string();
                                            current_user_data_vec[6] = "0".to_string();
                                            println!("step R-H 1");
                                            // 更新本地数据
                                            ONLINE_USERS.insert(
                                                user_id_clone.clone(),
                                                current_user_data_vec.join(","),
                                            );
                                            println!("step R-H 2");
                                            let user_id_clone_for = user_id_clone.clone();
                                            tokio::spawn(async move {
                                                // 更新redis数据
                                                if let Err(e) = update_client_redis_data(
                                                    user_id_clone_for.as_str(),
                                                    current_user_data_vec.join(","),
                                                )
                                                .await {
                                                    println!("更新redis数据失败：{:?} 用户id {}", e, user_id_clone_for);
                                                } else {
                                                    println!("通知挂断refuse线程，更新redis数据成功");
                                                }
                                            });
                                            println!("step R-H 3");
                                            // 对其发送CmdHangup，对发起方发送CmdRefuse
                                            // 发起方的isHost一定是1，找到该用户即可
                                            let host_user_id_now = {
                                                ONLINE_USERS
                                                .iter()
                                                .find(|entry| {
                                                    entry.value().split(',').collect::<Vec<_>>()[1]
                                                        == current_channel_id
                                                        && entry.value().split(',').collect::<Vec<_>>()[6]
                                                            == "1"
                                                })
                                                .map(|entry| entry.key().clone())
                                            };
                                            println!("step R-H 4");
                                            // 再找到对应的Sender
                                            let host_user_sender_which = if let Some(host_user_id) = host_user_id_now {
                                                CLIENT_SENDERS
                                                    .iter()
                                                    .find(|entry| entry.key().0 == host_user_id)
                                                    .map(|entry| entry.key().clone())
                                            } else {
                                                None
                                            };
                                            println!("step R-H 5");
                                            if let Some(host_user_sender_which) = host_user_sender_which {
                                                // 对其发送消息
                                                send_inside_message(
                                                    &host_user_sender_which,
                                                    &event_sender,
                                                    serde_json::json!({
                                                        "msgType": "CmdRefuse",
                                                        "fromID": user_id_clone,
                                                        "fromName": "Unknown",
                                                        "toID": &host_user_sender_which.0,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &host_user_sender_which.0,
                                                )
                                                .await;
                                            }
                                            println!("step R-H 6");
                                            // 向当前用户Sender发送CmdHangup要求其挂断
                                            let hangup_user_sender_which = if let Some(hangup_user_sender_which) =
                                            CLIENT_SENDERS.iter().find(|entry| entry.key().0 == user_id_clone) {
                                                Some(hangup_user_sender_which.key().clone())
                                            } else {
                                                None
                                            };
                                            println!("step R-H 7");
                                            if let Some(hangup_user_sender_which) = hangup_user_sender_which {
                                                // 对其发送消息
                                                send_inside_message(
                                                    &hangup_user_sender_which,
                                                    &event_sender,
                                                    serde_json::json!({
                                                        "msgType": "CmdHangup",
                                                        "fromID": "0",
                                                        "fromName": "Server",
                                                        "toID": from_id,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &user_id_clone.to_string(),
                                                ).await;
                                            }
                                            println!("step R-H 8");
                                            // 检查剩余频道人员数量
                                            let left_users: Vec<(String, String)> = {
                                                ONLINE_USERS.iter()
                                                .filter(|entry| entry.value().split(',').collect::<Vec<_>>()[1] == current_channel_id)
                                                .map(|entry| (entry.key().clone(), entry.value().clone())) // 克隆键和值
                                                .collect()
                                            };
                                            println!("step R-H 9");
                                            if left_users.len() <= 1 {
                                                let only_left_user = left_users.get(0);
                                                if let Some((user_id, user_data)) = only_left_user {
                                                    let mut user_data = user_data.split(',').collect::<Vec<&str>>();
                                                    {
                                                        println!("step R-H 10");
                                                        let user_is_host = user_data[1] == "1";
                                                        if user_is_host {
                                                            // 发起方
                                                            // 找到发起方的Sender
                                                            println!("step R-H 10-1");
                                                            let sender_or_not = {
                                                                CLIENT_SENDERS.iter().find(|entry| entry.key().0 == user_id.to_string())
                                                            };
                                                            if let Some(host_user_sender_which) = sender_or_not {
                                                                // 更新状态数据到OnlineUsers
                                                                user_data[0] = "idle";
                                                                user_data[1] = "";
                                                                user_data[6] = "0";
                                                                println!("step R-H 10-2");
                                                                ONLINE_USERS.insert(user_id.to_string(), user_data.join(","));
                                                                println!("step R-H 10-3");
                                                                let user_data_j = user_data.join(",");
                                                                let uid = user_id.clone();
                                                                tokio::spawn(async move {
                                                                    // 更新redis数据
                                                                    if let Err(e) = update_client_redis_data(&uid, user_data_j).await {
                                                                        println!("只有一人在会议中，更新redis数据失败: {}", e)
                                                                    } else {
                                                                        println!("只有一人在会议中，更新redis数据成功")
                                                                    }
                                                                });
                                                                println!("step R-H 10-4");
                                                                // 对其发送CmdCancelCall
                                                                send_inside_message(
                                                                    host_user_sender_which.key(),
                                                                    &event_sender,
                                                                    serde_json::json!({
                                                                        "msgType": "CmdCancelCall",
                                                                        "fromID": "0",
                                                                        "fromName": "Server",
                                                                        "toID": host_user_sender_which.key().0,
                                                                       "msgData": {}
                                                                    }).to_string(), user_id,
                                                                ).await;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                            println!("step R-H 11");
                                            tokio::spawn(async move {
                                                // 要求所有用户更新在线人员列表
                                                notify_all_clients_to_update_online_users().await;
                                            });
                                        }
                                        _ = cancel_rx.recv() => {
                                            // 线程被取消
                                            println!("用户已接听或者主动挂断或者通话被取消 {}", user_id_clone);
                                        }
                                    }
                                });
                            }
                        }
                    }
                }
            }
        }
        // 拒接电话
        "Refuse" => {
            println!("用户拒接电话 {}", from_id.to_string());
            // 当前用户拒接电话之后，直接给当前用户发送CmdHangup消息
            // 主动拒绝之后，也需要取消refuse_procedure_map的线程任务
            let target_sender_or_not = { refuse_procedure_map.get(from_id) };
            if let Some(ref ref_procedure_sender) = target_sender_or_not {
                if let Err(e) = ref_procedure_sender.send(()) {
                    println!("取消拒绝接听的线程队列失败：{:?}", e);
                } else {
                    println!("取消拒绝接听的线程队列成功");
                }
                drop(target_sender_or_not);
            }
            let user_data = { ONLINE_USERS.get(from_id) };
            let mut current_channel_id = "".to_string();
            // 给当前from_id用户发送CmdHangup，表示已经成功拒接了电话，并且修改了状态
            if let Some(ref current_user_data) = user_data {
                let mut current_user_data_vec = current_user_data
                    .split(',')
                    .map(|s| s.to_string())
                    .collect::<Vec<String>>();
                let hangup_refuse_message = serde_json::json!({
                    "msgType": "CmdHangup",
                    "fromID": "0",
                    "fromName": "Server",
                    "toID": from_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                .to_string();
                println!("step - Refuse - 0");
                current_channel_id = current_user_data_vec[1].clone();
                // 修改状态数据
                current_user_data_vec[0] = "idle".to_string();
                current_user_data_vec[1] = "".to_string();
                current_user_data_vec[6] = "0".to_string();
                println!("step - Refuse - 1");
                let current_user_data_joined = current_user_data_vec.join(",");
                drop(user_data);
                drop(current_user_data_vec);
                println!("step - Refuse - 2");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                let from_id_clone = from_id.to_string();
                println!("step - Refuse - 3");
                tokio::spawn(async move {
                    // 修改redis数据
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, current_user_data_joined).await
                    {
                        println!("更新redis数据失败：{:?} 用户id {}", e, from_id_clone);
                    } else {
                        println!("更新redis数据成功");
                    }
                });
                println!("step - Refuse - 4");
                // 直接给sender发送数据通知
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    hangup_refuse_message,
                    &from_id,
                )
                .await;
            }
            // 及时释放锁
            println!("step - Refuse - 5");
            // 注意，拒接电话只能有一个toID
            // 在这里，toID的状态数据由客户端维护
            // 找到对应toId的sender
            let sender_found = {
                CLIENT_SENDERS
                    .iter()
                    .find(|entry| entry.key().0 == client_message_data.to_id.to_string())
                    .map(|entry| entry.key().clone())
            };
            println!("step - Refuse - 6");
            if let Some(target_sender) = sender_found {
                // 直接发送CmdHangup数据
                let cmd_refuse_message = serde_json::json!({
                    "msgType": "CmdRefuse",
                    "fromID": from_id,
                    "fromName": "Server",
                    "toID": client_message_data.to_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                .to_string();
                send_inside_message(
                    &target_sender,
                    event_sender,
                    cmd_refuse_message,
                    &client_message_data.to_id,
                )
                .await;
            } else {
                println!("找不到toID对应的sender");
            }
            println!("step - Refuse - 7");
            // 检查同频道下是不是目前只有1人在线
            let current_all_chatters = {
                ONLINE_USERS
                    .iter()
                    .filter(|entry| entry.value().split(',').nth(1).unwrap() == current_channel_id)
                    .map(|entry| entry.key().to_string())
                    .collect::<Vec<String>>()
            };
            if current_all_chatters.len() == 1 {
                // 如果只有一个人在线，则直接向该sender发送CmdCancelCall即可
                let sender_found = {
                    CLIENT_SENDERS
                        .iter()
                        .find(|entry| entry.key().0 == current_all_chatters[0].to_string())
                        .map(|entry| entry.key().clone())
                };
                if let Some(sender) = sender_found {
                    println!("step - Refuse - 7.75 - alone");
                    // 直接发送CmdCancelCall
                    let cmd_cancel_call_message = serde_json::json!({
                        "msgType": "CmdCancelCall",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": current_all_chatters[0],
                        "msgData": {
                            "channelId": "",
                            "rtcToken": ""
                        }
                    })
                    .to_string();
                    send_inside_message(
                        &sender,
                        event_sender,
                        cmd_cancel_call_message,
                        &current_all_chatters[0],
                    )
                    .await;
                    println!("step - Refuse - 7.755 - alone");
                    // 修改状态数据
                    let target_id = current_all_chatters[0].to_string();
                    drop(current_all_chatters);
                    // 获取ONLINE_USERS
                    println!("step - Refuse - 7.758 - alone");
                    let target_ud = ONLINE_USERS.get(&target_id);
                    if let Some(target_ud) = target_ud {
                        let mut target_ud_vec: Vec<&str> = target_ud.split(',').collect();
                        target_ud_vec[0] = "idle";
                        target_ud_vec[1] = "";
                        target_ud_vec[6] = "0";
                        let target_ud_joined = target_ud_vec.join(",");
                        println!("step - Refuse - 7.7599 - alone");
                        drop(target_ud_vec);
                        drop(target_ud);
                        ONLINE_USERS.insert(target_id.clone(), target_ud_joined.clone());
                        println!("step - Refuse - 7.759999 - alone");
                        tokio::spawn(async move {
                            // 修改redis数据
                            if let Err(e) =
                                update_client_redis_data(&target_id, target_ud_joined).await
                            {
                                println!("更新redis数据失败：{:?} 用户id {}", e, target_id);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                    }
                }
            }
            println!("step - Refuse - 7.8");
            tokio::spawn(async move {
                // 要求所有用户更新在线人员列表
                notify_all_clients_to_update_online_users().await;
            });
            println!("step - Refuse - 8");
        }
        // 主持人结束通话
        "EndMeeting" => {
            println!("step - EndMeeting - 1 收到挂断会议通知！");
            let user_found = { ONLINE_USERS.get(from_id) };
            // 要求判断是否为主持人，只有主持人可以结束通话
            if let Some(ref current_user_data) = user_found {
                let current_user_data_vec: Vec<&str> = current_user_data.split(',').collect();
                // 如果不是主持人，不允许发送该消息
                println!("step - EndMeeting - 1.1 判断是否为主持人");
                if current_user_data_vec[6] != "1" {
                    println!("step - EndMeeting - 1.2 不是主持人");
                    // 发送错误信息
                    let error_message = serde_json::json!({
                        "msgType": "Error",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": "只有主持人可以结束会议！"
                    })
                    .to_string();
                    send_inside_message(
                        &target_sender_which,
                        event_sender,
                        error_message,
                        &from_id,
                    )
                    .await;
                    println!("step - EndMeeting - 1.3 不是主持人消息发送完毕");
                    drop(current_user_data_vec);
                    drop(user_found);
                } else {
                    // 如果是主持人，则向所有人发送CmdEndMeeting消息，并且清理所有数据
                    // 提取出目前channel_id
                    println!("step - EndMeeting - 1.4 判断是主持人");
                    let channel_id = current_user_data_vec[1].to_string();
                    if channel_id != "" {
                        // 从ONLINE_USERS中过滤筛选出所有与channelID相同的用户id集合
                        println!("step - EndMeeting - 1.5 判断channel_id");
                        drop(user_found);
                        let users_to_notify = {
                            ONLINE_USERS
                                .iter()
                                .filter(|entry| {
                                    entry.value().split(",").collect::<Vec<_>>()[1] == channel_id
                                })
                                .map(|entry| entry.key().to_string())
                                .collect::<Vec<String>>()
                        };
                        let clone_for_users_to_notify = users_to_notify.clone();
                        println!("step - EndMeeting - 1.6 判断channel_id 遍历循环");
                        // 再根据上面的id，找到所有sender，并且修改所有对应ONLINE_USERS数据
                        for user_id_current_chat in users_to_notify {
                            let user_d = { ONLINE_USERS.get(&user_id_current_chat) };
                            if let Some(current_user_data) = user_d {
                                let mut current_user_data_vec: Vec<&str> =
                                    current_user_data.split(',').collect();
                                current_user_data_vec[0] = "idle";
                                current_user_data_vec[1] = "";
                                current_user_data_vec[6] = "0";
                                let current_user_data_joined = current_user_data_vec.join(",");
                                println!("step - EndMeeting - 1.7 判断channel_id 遍历循环 1");
                                drop(current_user_data_vec);
                                drop(current_user_data);
                                ONLINE_USERS.insert(
                                    user_id_current_chat.to_string(),
                                    current_user_data_joined.clone(),
                                );
                                println!("step - EndMeeting - 1.8 判断channel_id 遍历循环 2");
                                tokio::spawn(async move {
                                    // 更新对应redis数据
                                    if let Err(e) = update_client_redis_data(
                                        user_id_current_chat.as_str(),
                                        current_user_data_joined,
                                    )
                                    .await
                                    {
                                        println!(
                                            "更新redis数据失败：{:?} 用户id {}",
                                            e, user_id_current_chat
                                        );
                                    } else {
                                        println!("更新redis数据成功");
                                    }
                                });
                                println!("玄幻完毕");
                            }
                        }
                        println!("step - EndMeeting - 1.9 发送消息 遍历循环 3");
                        for user_id in clone_for_users_to_notify {
                            if let Some(target_sender) = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == user_id)
                                .map(|entry| entry.key().clone())
                            {
                                // 找到这个sender标记符之后，发送消息CmdEndMeeting
                                println!("step - EndMeeting 消息发送给 {:?}", &(target_sender));
                                send_inside_message(
                                    &target_sender,
                                    event_sender,
                                    serde_json::json!({
                                        "msgType": "CmdEndMeeting",
                                        "fromID": "0",
                                        "fromName": "Server",
                                        "toID": user_id,
                                        "msgData": {
                                            "channelId": "",
                                            "rtcToken": ""
                                        }
                                    })
                                    .to_string(),
                                    &user_id,
                                )
                                .await;
                            }
                        }
                        tokio::spawn(async move {
                            // 发送完毕之后，要求更新所有用户数据
                            notify_all_clients_to_update_online_users().await;
                        });
                    }
                }
            }
        }
        // 通话过程中挂断
        "Hangup" => {
            let user_d = { ONLINE_USERS.get(from_id) };
            // 挂断通话不是拒接，不需要处理refuse_procedure_map中的线程消息
            if let Some(current_user_data) = user_d {
                // 直接修改对应数据即可
                let mut current_user_data_vec: Vec<&str> = current_user_data.split(',').collect();
                let is_current_user_host = current_user_data_vec[6] == "1";
                let current_chatting_channel_id = current_user_data_vec[1];
                current_user_data_vec[0] = "idle";
                current_user_data_vec[1] = "";
                // 无论是否是主持人，都需要发送对应消息，更新数据以及同步redis
                current_user_data_vec[6] = "0";
                let current_user_data_joined = current_user_data_vec.join(",");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                let from_id_clone = from_id.to_string();
                tokio::spawn(async move {
                    // 更新redis数据
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, current_user_data_joined).await
                    {
                        println!("更新redis数据失败：{:?} 用户id {}", e, from_id_clone);
                    } else {
                        println!("更新redis数据成功");
                    }
                });
                // 直接向其发送CmdHangup即可
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    serde_json::json!({
                        "msgType": "CmdHangup",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": {
                            "channelId": "",
                            "rtcToken": ""
                        }
                    })
                    .to_string(),
                    &from_id,
                )
                .await;
                // 如果当前用户是主持人，主持人退出之后，需要在剩下的用户中决定新主持人
                if is_current_user_host {
                    // 过滤出剩下来的channelID同一的在线用户，并且整合为Vec集合
                    let remaining_users: Vec<String> = {
                        ONLINE_USERS
                            .iter()
                            .filter(|entry| {
                                let v_vec: Vec<&str> = entry.value().split(',').collect();
                                v_vec[1] == current_chatting_channel_id
                            })
                            .map(|entry| entry.key().to_string())
                            .collect()
                    };
                    // 没有人参与会议了，直接退出即可
                    if remaining_users.len() == 0 {
                        println!("当前频道没有人员，请重新发起通话");
                    }
                    // 如果余留下的用户数量仅为1，则开启线程，20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令
                    // 如果大于2人，则需要判断主持人
                    else {
                        // 无论如何，都要向他们发送CmdLeave消息
                        for user_id in remaining_users.iter() {
                            let sender_found = {
                                CLIENT_SENDERS
                                    .iter()
                                    .find(|entry| entry.key().0 == *user_id)
                                    .map(|entry| entry.key().clone())
                            };
                            if let Some(target_sender) = sender_found {
                                // 找到这个sender标记符之后，发送消息CmdEndMeeting
                                let cmd_leave_message = serde_json::json!({
                                    "msgType": "CmdLeave",
                                    "fromID": &from_id,
                                    "fromName": "Unknown",
                                    "toID": user_id,
                                    "msgData": {}
                                })
                                .to_string();
                                send_inside_message(
                                    &target_sender,
                                    event_sender,
                                    cmd_leave_message,
                                    user_id,
                                )
                                .await;
                            }
                        }
                        if remaining_users.len() == 1 {
                            let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                            channel_hangup_procedure_map
                                .insert(current_chatting_channel_id.to_string(), cancel_tx);
                            let channel_id_clone = current_chatting_channel_id.to_string();
                            let ev_clone = event_sender.clone();

                            tokio::spawn(async move {
                                let remaining_users_clone = remaining_users.clone();
                                tokio::select! {
                                    _ = tokio::time::sleep(Duration::from_secs(20)) => {
                                        println!("20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令 {:?}", remaining_users_clone);
                                        if let Some(remaining_user_id) = remaining_users_clone.get(0) {
                                            let remaining_user_id_clone = remaining_user_id.clone(); // 克隆 remaining_user_id

                                            let remain_user_d = ONLINE_USERS.get(&remaining_user_id_clone);
                                            if let Some(remain_user_info_data) = remain_user_d {
                                                let sender_d = CLIENT_SENDERS
                                                   .iter()
                                                   .find(|entry| entry.key().0 == remaining_user_id_clone)
                                                   .map(|entry| entry.key().clone());

                                                if let Some(target_sender_which) = sender_d {
                                                    let mut user_info_data_for_remained = remain_user_info_data.split(',').collect::<Vec<&str>>();
                                                    user_info_data_for_remained[0] = "idle";
                                                    user_info_data_for_remained[1] = "";
                                                    user_info_data_for_remained[6] = "0";
                                                    let user_info_data_for_remained_joined = user_info_data_for_remained.join(",");

                                                    // 插入克隆后的 remaining_user_id_clone
                                                    ONLINE_USERS.insert(remaining_user_id_clone.clone(), user_info_data_for_remained_joined.clone());

                                                    // 内部 tokio::spawn 使用克隆后的 remaining_user_id_clone
                                                    let clone_for_redis = remaining_user_id_clone.clone();
                                                    tokio::spawn(async move {
                                                        if let Err(e) = update_client_redis_data(&clone_for_redis, user_info_data_for_remained_joined).await {
                                                            println!("更新redis数据失败：{:?} 用户id {}", e, clone_for_redis);
                                                        } else {
                                                            println!("更新redis数据成功");
                                                        }
                                                    });

                                                    send_inside_message(
                                                        &target_sender_which,
                                                        &ev_clone,
                                                        serde_json::json!({
                                                            "msgType": "CmdHangup",
                                                            "fromID": "0",
                                                            "fromName": "Server",
                                                            "toID": remaining_user_id_clone,
                                                            "msgData": {
                                                                "channelId": "",
                                                                "rtcToken": ""
                                                            }
                                                        }).to_string(),
                                                        &remaining_user_id_clone,
                                                    ).await;

                                                    tokio::spawn(async move {
                                                        notify_all_clients_to_update_online_users().await;
                                                    });
                                                }
                                            }
                                        }
                                    }
                                    _ = cancel_rx.recv() => {
                                        println!("收到取消信号，有新的人员加入到会议，会议频道号 {} 结束线程", channel_id_clone);
                                    }
                                }
                            });
                        }
                        // 如果余下用户数量超过2人及以上，则需要判断应该让谁让主持人
                        // 如果余下用户数量超过2人及以上，则需要判断应该让谁成为主持人
                        else {
                            // 分组：1.筛选出id在HOST_ENABLED_ID_SET中的用户，即允许成为主持人的用户，2.筛选出id不在HOST_ENABLED_ID_SET中的用户，即不允许成为主持人的用户
                            // 注意，这里只需要修改一下状态数据即可，不用单独发送消息
                            let (allowed_users, disallowed_users): (Vec<String>, Vec<String>) =
                                remaining_users.into_iter().partition(|user_id| {
                                    HOST_ENABLED_ID_SET.contains(user_id.as_str())
                                });
                            let allowed_users_clone = allowed_users.clone(); // 克隆 allowed_users
                            if !allowed_users_clone.is_empty() {
                                // 让第一位id的用户成为主持人
                                if let Some(allowed_user_id) = allowed_users_clone.get(0) {
                                    let allowed_user_id_clone = allowed_user_id.clone(); // 克隆 allowed_user_id
                                    let data_f = { ONLINE_USERS.get(&allowed_user_id_clone) };
                                    if let Some(ref allowed_user_info_data) = data_f {
                                        let mut user_info_data_for_allowed = allowed_user_info_data
                                            .split(',')
                                            .map(|s| s.to_string())
                                            .collect::<Vec<String>>();
                                        user_info_data_for_allowed[6] = "1".to_string();
                                        // 修改online数据信息
                                        let user_info_data_for_allowed_joined =
                                            user_info_data_for_allowed.join(",");
                                        drop(data_f);
                                        ONLINE_USERS.insert(
                                            allowed_user_id_clone.clone(),
                                            user_info_data_for_allowed_joined.clone(),
                                        );
                                        tokio::spawn(async move {
                                            if let Err(e) = update_client_redis_data(
                                                &allowed_user_id_clone,
                                                user_info_data_for_allowed_joined,
                                            )
                                            .await
                                            {
                                                println!(
                                                    "更新redis数据失败：{:?} 用户id {}",
                                                    e, allowed_user_id_clone
                                                );
                                            } else {
                                                println!("更新redis数据成功");
                                            }
                                        });
                                    }
                                }
                            } else if !disallowed_users.is_empty() {
                                // 让第一位
                                let disallowed_users_clone = disallowed_users.clone(); // 克隆 disallowed_users
                                if let Some(disallowed_user_id) = disallowed_users_clone.get(0) {
                                    let disallowed_user_id_clone = disallowed_user_id.clone(); // 克隆 disallowed_user_id
                                    let d_f = { ONLINE_USERS.get(&disallowed_user_id_clone) };
                                    if let Some(ref disallowed_user_info_data) = d_f {
                                        let mut current_host_data = disallowed_user_info_data
                                            .split(',')
                                            .map(|s| s.to_string())
                                            .collect::<Vec<String>>();
                                        current_host_data[6] = "1".to_string();
                                        let current_host_data_joined = current_host_data.join(",");
                                        drop(d_f);
                                        // 更新ONLINE_USERS数据
                                        ONLINE_USERS.insert(
                                            disallowed_user_id_clone.clone(),
                                            current_host_data_joined.clone(),
                                        );
                                        tokio::spawn(async move {
                                            if let Err(e) = update_client_redis_data(
                                                &disallowed_user_id_clone,
                                                current_host_data_joined,
                                            )
                                            .await
                                            {
                                                println!(
                                                    "更新redis数据失败：{:?} 用户id {}",
                                                    e, disallowed_user_id_clone
                                                );
                                            } else {
                                                println!("更新redis数据成功");
                                            }
                                        });
                                    }
                                }
                            }
                        }
                    }
                }
                tokio::spawn(async move {
                    // 无论如何都要发送通知在线人员更新
                    notify_all_clients_to_update_online_users().await;
                });
            }
        }
        // 接听通话，这个是目前最重要的逻辑部分
        // 在这里要处理如下事项
        // 1.取消20s内挂断某个用户的线程；
        // 2.若加入的频道号之前只有一个用户，那么目前频道是超过2人的，那么要取消20s内退出的线程；
        // 3.修改状态并且广播出去
        // 4.决定主持人。一般来说，谁发起了这个频道的人，并且具备成为主持人的资格，那么就直接成为主持人，否则就让第一个加入的人成为主持人。
        "Connect" => {
            println!(
                "收到客户端Connect消息连接 {} 频道信息 {:?}",
                &from_id, &client_message_data
            );
            // 发送connect消息之时，默认以toID用户的channelID作为channelID
            let to_id_user = &client_message_data.to_id;
            // 找到对应to_id用户的数据
            println!("Step Connect-1");
            let to_id_user_data_here = { ONLINE_USERS.get(to_id_user) };
            if let Some(to_id_user_data) = to_id_user_data_here {
                // 如果存在，则获取这个channel_id
                let data_split = to_id_user_data
                    .split(',')
                    .map(|s| s.to_string())
                    .collect::<Vec<String>>();
                // 提前释放资源，释放对应锁
                println!("Step Connect-1.1");
                drop(to_id_user_data);
                let target_channel_id = data_split[1].clone();
                if !target_channel_id.is_empty() {
                    // 这里的from_id就是被呼叫方，to_id就是呼叫方
                    // 首先先取消掉所有的挂断from_id的任务
                    println!("Step Connect-1.2");
                    let hangup_p = { refuse_procedure_map.get(from_id) };
                    if let Some(hangup_personnel) = hangup_p {
                        if let Err(e) = hangup_personnel.send(()) {
                            println!("挂断personnel的任务终结失败 {:?}", e)
                        } else {
                            println!("挂断personnel的任务已经发送成功")
                        }
                    }
                    println!("Step Connect-1.3");
                    // 检查一下是否存在挂断channel的任务，如果存在，对齐发送消息，然后关闭
                    let hangup_p = { channel_hangup_procedure_map.get(&target_channel_id) };
                    if let Some(hangup_channel_task) = hangup_p {
                        if let Err(e) = hangup_channel_task.send(()) {
                            println!("挂断channel的任务终结失败 {:?}", e)
                        } else {
                            println!("挂断channel的任务已经发送成功")
                        }
                    }
                    println!("Step Connect-1.4");
                    // 如果当前channel已经有主持人，则不做处理
                    let already_had_host = {
                        ONLINE_USERS
                            .iter()
                            .filter(|entry| {
                                entry.value().split(",").collect::<Vec<_>>()[1] == target_channel_id
                            })
                            .any(|entry| entry.value().split(",").collect::<Vec<_>>()[6] == "1")
                    };
                    println!("Step Connect-1.5");
                    if already_had_host {
                        println!("Step Connect-1.6");
                        println!("当前channel已经有主持人，直接返回 {}", target_channel_id)
                    } else {
                        println!("当前channel没有主持人，开始处理 {}", target_channel_id);
                        // 首先判断发起方，也就是channel_id用_分割之后的第一个元素，是否具备成为主持人的资格
                        println!("Step Connect-1.8");
                        let last_remaining_id_set = target_channel_id
                            .split("_")
                            .map(|s| s.to_string())
                            .collect::<Vec<_>>();
                        if let Some(channel_id_first_element) = last_remaining_id_set.get(0) {
                            println!("Step Connect-2");
                            if HOST_ENABLED_ID_SET.contains(&channel_id_first_element.to_string()) {
                                // 具备成为主持人资格，直接修改其对应数据即可
                                let online_d =
                                    { ONLINE_USERS.get(&channel_id_first_element.to_string()) };
                                if let Some(ref host_data) = online_d {
                                    let mut host_data_vec =
                                        host_data.split(",").collect::<Vec<_>>();
                                    host_data_vec[6] = "1";
                                    let host_data_joined = host_data_vec.join(",");
                                    println!("Step Connect-3");
                                    drop(online_d);
                                    println!("Step Connect-4");
                                    // 直接再次更新online_users
                                    ONLINE_USERS.insert(
                                        channel_id_first_element.to_string(),
                                        host_data_joined.clone(),
                                    );
                                    println!("Step Connect-5");
                                    let the_first_for = channel_id_first_element.to_string();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(
                                            &the_first_for,
                                            host_data_joined.clone(),
                                        )
                                        .await
                                        {
                                            println!(
                                                "更新redis数据失败：{:?} 用户id {} 成为主持人",
                                                e, the_first_for
                                            )
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                    println!("Step Connect-6");
                                }
                            } else {
                                println!("Step Connect-7");
                                // 获取所有同channel_id的用户，在ONLINE_USERS中寻找第一个满足条件的id
                                // 记录是否找到主持人对应id
                                let all_in_group_chatters_id: Vec<String> = {
                                    ONLINE_USERS
                                        .iter()
                                        .filter(|entry| {
                                            entry.value().split(",").collect::<Vec<_>>()[1]
                                                == target_channel_id
                                        })
                                        .map(|entry| entry.key().clone())
                                        .collect::<Vec<String>>()
                                };
                                // 在all_in_group_chatters_id中，找寻第一个在HOST_ENABLED_ID_SET中的id
                                let first_host_id = all_in_group_chatters_id.iter()
                                    .find(|id| HOST_ENABLED_ID_SET.contains(&id.to_string()));
                                let mut id = "";
                                if let Some(target_id) = first_host_id {
                                    id = target_id;
                                } else {
                                    // 若是空的，则直接取值第一个即可
                                    if let Some(first_element) =
                                        all_in_group_chatters_id.first()
                                    {
                                        id = first_element;
                                    }
                                }
                                println!("Step Connect-7.5 找到具备成为主持人的host-id");
                                let hd = { ONLINE_USERS.get(id) };
                                if let Some(ref host_data) = hd {
                                    let mut host_data_vec =
                                        host_data.split(",").collect::<Vec<_>>();
                                    host_data_vec[6] = "1";
                                    println!("Step Connect-8");
                                    // 更新对应数据
                                    let host_data_joined = host_data_vec.join(",");
                                    println!("Step Connect-9");
                                    drop(hd);
                                    ONLINE_USERS
                                        .insert(id.to_string(), host_data_joined.clone());
                                    println!("Step Connect-9.1");
                                    let host_id_clone = id.to_string();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(
                                            &host_id_clone,
                                            host_data_joined.clone(),
                                        )
                                            .await
                                        {
                                            println!(
                                                "更新redis数据失败：{:?} 用户id {} 成为主持人",
                                                e, host_id_clone
                                            )
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                    println!("Step Connect-10");
                                }
                            }
                        }
                    }
                    // 设置主持人之后，需要对双方发送CmdConnect消息
                    println!("Step Connect-11");
                    // 注意，用户接听之后，是要对同channel_id下的所有其余状态不为calling的用户发送
                    let all_not_calling_users = {
                        ONLINE_USERS
                            .iter()
                            .filter(|entry| {
                                let entry_split = entry.value().split(",").collect::<Vec<_>>();
                                entry_split[1] == target_channel_id && entry_split[0] != "calling"
                            })
                            .map(|entry| entry.key().clone())
                           .collect::<Vec<String>>()
                    };
                    println!("Step Connect-11.5 当前所有channel用户 {:?}和channelID {}", all_not_calling_users,target_channel_id.to_string());
                    // 遍历上述结构，然后针对他们均发送CmdConnect消息
                    for user_id in all_not_calling_users {
                        // 找到对应sender
                        let target_sender_which = {
                            CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == user_id)
                                .map(|entry| entry.key().clone())
                        };
                        if let Some(target_sender_which) = target_sender_which {
                            let json_str = serde_json::json!({
                                "msgType": "CmdConnect",
                                "msgData": {
                                    "channelID": target_channel_id,
                                    "rtcToken": ""
                                },
                                "fromID" : "0",
                                "fromName": "Server",
                                "toID": user_id
                            })
                            .to_string();
                            // 发送消息
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                target_sender_which.clone(),
                                json_str,
                                false,
                            )) {
                                println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                            } else {
                                println!("发送给用户id {} 独立消息成功 CmdConnect", user_id);
                            }
                        }
                    }
                    tokio::spawn(async move {
                        // 最后广播所有用户更新状态
                        notify_all_clients_to_update_online_users().await;
                    });
                    println!("Step Connect-11");
                } else {
                    if let Err(e) = event_sender.send(Event::SendClientMessage(
                        target_sender_which.clone(),
                        serde_json::json!({
                            "msgType": "Error",
                            "msgData": "对方数据出现异常，缺少channelID",
                            "fromID" : "0",
                            "fromName": "Server",
                            "toID": from_id
                        })
                        .to_string(),
                        false,
                    )) {
                        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                    }
                }
            } else {
                if let Err(e) = event_sender.send(Event::SendClientMessage(
                    target_sender_which.clone(),
                    serde_json::json!({
                        "msgType": "Error",
                        "msgData": "必须传递to_id",
                        "fromID" : "0",
                        "fromName": "Server",
                        "toID": from_id
                    })
                    .to_string(),
                    false,
                )) {
                    println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                }
            }
        }
        // 处理静音
        "Mute" => {}
        _ => {}
    }
}
